import sys
import os
import time



system_directory=os.getcwd()
imagery_directory=os.path.join(system_directory, r'imagery')
imagery_directory_content=os.listdir(imagery_directory)
mask_path=os.path.join(system_directory,r'shape',r'mask.shp')



def clip(date_folder):
    print('')
    print('Clipping file.'+date_folder)
    folder_found=False
    imagery_directory_content=os.listdir(imagery_directory)             #refresh directory content
    for name in imagery_directory_content:                            #search the folder for a given date
        if str(name) == str(date_folder):
            folder_found=True
    if folder_found == True:
        date_folder_directory=os.path.join(imagery_directory, str(date_folder))
        print('')
        print('Clip script. Starting....')
        print('Working on ',date_folder_directory)
        date_folder_content=os.listdir(date_folder_directory)
        for files in date_folder_content:                            #if found, for each jp2 file does clip with the mask
            files_parts=files.split('.')
            files_status=files_parts[0].split('_')
            if files_parts[-1]=='jp2' and files_status[-1]!='clip' and files_status[-1]!='NBR':
                input_file_path=os.path.join(date_folder_directory,str(files))
                output_file_name=str(files_parts[0])+"_clip.tif"
                output_file_path=os.path.join(date_folder_directory,output_file_name)
                print('Input file: '+input_file_path)
                print("Output_file: "+output_file_path)
                print('')
                gdal_call='gdalwarp -ot UInt16 -dstnodata 0 -of GTiff -tr 10.0 -10.0 -tap -cutline "'+str(mask_path)+'" "'+str(input_file_path)+'" "'+str(output_file_path)+'"'
                os.system(gdal_call)
                os.remove(input_file_path)
                print ("Clip done")


    else:
        print ('Date folder not found.')



def get_tiles(date_folder):
    imagery_directory_content=os.listdir(imagery_directory)
    if date_folder in imagery_directory_content:    #search the folder for a given date
        date_folder_directory=os.path.join(imagery_directory, str(date_folder))
        date_folder_content=os.listdir(date_folder_directory)
        tile_list=list()
        for files in date_folder_content:                            #creates a list with every Tile
            files_parts=files.split('.')
            files_status=files_parts[0].split('_')
            if files_status[0] not in tile_list and len(files_status[0])==6:
                tile_list.extend([files_status[0]])
        return tile_list
    else:
        return None


def NBR(date_folder):
    print('')
    print('Calculating NBR index.')
    imagery_directory_content=os.listdir(imagery_directory)
    if date_folder in imagery_directory_content:    #search the folder for a given date
        date_folder_directory=os.path.join(imagery_directory, str(date_folder))
        date_folder_content=os.listdir(date_folder_directory)
        tile_list=list()
        for files in date_folder_content:                            #creates a list with every Tile
            files_parts=files.split('.')
            files_status=files_parts[0].split('_')
            if files_status[0] not in tile_list:
                tile_list.extend([files_status[0]])
        print ('Tiles Found: ',tile_list)

        for tile_name in tile_list:                                 #find NBR bands and call gdal to calculate
            B08=None
            B12=None
            CLOUDS=None

            for files in date_folder_content:
                files_parts=files.split('.')
                files_status=files_parts[0].split('_')

                if files_status[-2]=='B08' and files_status[0]==tile_name and files_status[-1]=='clip' and files_parts[-1]=='tif':
                    B08=os.path.join(date_folder_directory, files)
                    print ("B08 file: ",B08)
                if files_status[-2]=='B12' and files_status[0]==tile_name and files_status[-1]=='clip' and files_parts[-1]=='tif':
                    B12=os.path.join(date_folder_directory, files)
                    print ("B12 file: ",B12)
                if files_status[-1]=='B00' and files_status[0]==tile_name and files_parts[-1]=='gml':
                    CLOUDS=os.path.join(date_folder_directory, files)
                    print ("Cloud mask file: ",CLOUDS)

            if B08 is None or B12 is None:
                print('Files not found')
            else:
                NBR_file_name=tile_name+"_"+str(date_folder)+"_NBR.tif"
                NBR_file_path=os.path.join(date_folder_directory,NBR_file_name)
                if NBR_file_name in date_folder_content:
                    print('Output file already exists.')
                else:
                    gdal_calc_call='gdal_calc.py --type="Float32" -A "'+str(B08)+'" -B "'+str(B12)+'" --outfile="'+str(NBR_file_path)+'" --calc "(A.astype(float32)-B.astype(float32))/(A.astype(float32)+B.astype(float32))"'
                    os.system(gdal_calc_call)
                    if CLOUDS!=None:
                        gdal_call='gdal_rasterize -at -burn 3.40282e+38 -where "maskType=\'OPAQUE\'" "'+CLOUDS+'" "'+NBR_file_path+'"'
                        print('Applying cloud mask.')
                        os.system(gdal_call)
                    print ('Output File: ',NBR_file_path)

    else:
        print ('Date folder not found.')




def difference_NBR(date_last , date_first):                     #for two given dates and a tile name, calculates the NBR difference index
    print('')
    print ('Calculating NBR index difference.')
    imagery_directory_content_rev=sorted(os.listdir(imagery_directory))
    imagery_directory_content=list(reversed(imagery_directory_content_rev))
    if date_first in imagery_directory_content and date_last in imagery_directory_content:
        date_first_directory=os.path.join(imagery_directory, str(date_first))
        date_last_directory=os.path.join(imagery_directory, str(date_last))
        date_first_content=os.listdir(date_first_directory)
        date_last_content=os.listdir(date_last_directory)

        first_tiles=get_tiles(date_first)
        last_tiles=get_tiles(date_last)
        tiles_found=set(first_tiles).intersection(last_tiles)


        for tile_name in tiles_found:

            first_file_flag=False
            last_file_flag=False

            print('Calculating Tile: '+tile_name)

            for files in date_first_content:                     #check both folders looking for the given tiles
                file_name=files.split('.')[0]
                file_format=files.split('.')[-1]
                file_name_parts=file_name.split('_')
                if file_name_parts[0]==tile_name and file_name_parts[-1]=='NBR' and file_format=='tif':
                    first_file_flag=True
                    first_file_path=os.path.join(date_first_directory, files)
                if file_name_parts[-1]=='B00' and file_name_parts[0]==tile_name and file_format=='gml':
                    CLOUDS_first=os.path.join(date_first_directory, files)
                    print ("Cloud mask file: ",CLOUDS_first)



            for files in date_last_content:
                file_name=files.split('.')[0]
                file_format=files.split('.')[-1]
                file_name_parts=file_name.split('_')
                if file_name_parts[0]==tile_name and file_name_parts[-1]=='NBR' and file_format=='tif':
                    last_file_flag=True
                    last_file_path=os.path.join(date_last_directory, files)
                if file_name_parts[-1]=='B00' and file_name_parts[0]==tile_name and file_format=='gml':
                    CLOUDS_last=os.path.join(date_last_directory, files)
                    print ("Cloud mask file: ",CLOUDS_last)


            if first_file_flag==True and last_file_flag==True:         #if correct tiles are found, it calculates the nbr difference
                print ('First file: ',first_file_path)
                print ('Last file: ',last_file_path)
                output_diff_NBR_name=str(tile_name)+'_'+str(date_last)+'_'+str(date_first)+'_DIF_NBR.tif'
                output_diff_NBR_path=os.path.join(date_last_directory,output_diff_NBR_name)
                if output_diff_NBR_name in date_last_content:
                    print('Output file already exists.')
                else:
                    print ('Output file: ',output_diff_NBR_path)
                    print ('')
                    print ('Calling GDAL...')
                    print ('')
                    gdal_call='gdal_calc.py -A "'+str(first_file_path)+'" -B "'+str(last_file_path)+'" --outfile="'+str(output_diff_NBR_path)+'" --calc "A-B"'
                    os.system(gdal_call)

                    if CLOUDS_first!=None:
                        gdal_call='gdal_rasterize -at -burn 200 -where "maskType=\'OPAQUE\'" "'+CLOUDS_first+'" "'+output_diff_NBR_path+'"'
                        print('Applying cloud mask.')
                        os.system(gdal_call)

                    if CLOUDS_last!=None:
                        gdal_call='gdal_rasterize -at -burn 200 -where "maskType=\'OPAQUE\'" "'+CLOUDS_last+'" "'+output_diff_NBR_path+'"'
                        print('Applying cloud mask.')
                        os.system(gdal_call)

            else:
                if first_file_flag==False and last_file_flag==False:
                    print ('Files not found')
                elif first_file_flag==False:
                    print ('First file not found')
                else:
                    print ('Last file not found')
                return("NOT_FOUND")


    elif date_first=='0' and date_last in imagery_directory_content:
        #get folder position on the list and the last position on the list
        folder_position=imagery_directory_content.index(date_last)+1

        folder_max=len(imagery_directory_content)-1
        date_last_directory=os.path.join(imagery_directory, str(date_last))
        date_last_content=os.listdir(date_last_directory)
        last_tiles=get_tiles(date_last)

        print('Calculating NBR')
        print('')
        print('Tiles in '+str(date_last)+': '+str(last_tiles))
        print('Folder position: '+str(folder_position-1)+'. Last position: '+str(folder_max))
        print('Available folders: '+str(imagery_directory_content))

        #starts checking the next folder (closest in the past). Removes found elements from the list.
        for folders in imagery_directory_content[folder_position:]:
            if len(last_tiles) == 0:
                break
            first_tiles=get_tiles(folders)
            tiles_found=list(set(first_tiles).intersection(last_tiles))
            date_first_directory=os.path.join(imagery_directory, str(folders))
            date_first_content=os.listdir(date_first_directory)

            print('')
            print('Match Tiles in '+folders+': '+str(tiles_found))

            for tile_name in tiles_found:         #for the found common tiles, performs the NBR_difference

                first_file_flag=False
                last_file_flag=False

                print('Calculating Tile: '+tile_name)

                for files in date_first_content:                     #check both folders looking for the given tiles
                    file_name=files.split('.')[0]
                    file_format=files.split('.')[-1]
                    file_name_parts=file_name.split('_')
                    if file_name_parts[0]==tile_name and file_name_parts[-1]=='NBR' and file_format=='tif':
                        first_file_flag=True
                        first_file_path=os.path.join(date_first_directory, files)
                    if file_name_parts[-1]=='B00' and file_name_parts[0]==tile_name and file_format=='gml':
                        CLOUDS_first=os.path.join(date_first_directory, files)
                        print ("Cloud mask file: ",CLOUDS_first)




                for files in date_last_content:
                    file_name=files.split('.')[0]
                    file_format=files.split('.')[-1]
                    file_name_parts=file_name.split('_')
                    if file_name_parts[0]==tile_name and file_name_parts[-1]=='NBR' and file_format=='tif':
                        last_file_flag=True
                        last_file_path=os.path.join(date_last_directory, files)
                    if file_name_parts[-1]=='B00' and file_name_parts[0]==tile_name and file_format=='gml':
                        CLOUDS_last=os.path.join(date_last_directory, files)
                        print ("Cloud mask file: ",CLOUDS_last)


                if first_file_flag==True and last_file_flag==True:         #if correct tiles are found, it calculates the nbr difference
                    print ('First file: ',first_file_path)
                    print ('Last file: ',last_file_path)
                    output_diff_NBR_name=str(tile_name)+'_'+str(date_last)+'_'+str(folders)+'_DIF_NBR.tif'
                    output_diff_NBR_path=os.path.join(date_last_directory,output_diff_NBR_name)
                    print ('Output file: ',output_diff_NBR_path)
                    if output_diff_NBR_name in date_last_content:
                        print('Output file already exists.')
                    else:

                        print ('')
                        print ('Calling GDAL...')
                        print ('')
                        gdal_call='gdal_calc.py -A "'+str(first_file_path)+'" -B "'+str(last_file_path)+'" --outfile="'+str(output_diff_NBR_path)+'" --calc "A-B" '
                        os.system(gdal_call)


                        if CLOUDS_first!=None:
                            gdal_call='gdal_rasterize -at -burn 200 -where "maskType=\'OPAQUE\'" "'+CLOUDS_first+'" "'+output_diff_NBR_path+'"'
                            print('Applying cloud mask.')
                            os.system(gdal_call)

                        if CLOUDS_last!=None:
                            gdal_call='gdal_rasterize -at -burn 200 -where "maskType=\'OPAQUE\'" "'+CLOUDS_last+'" "'+output_diff_NBR_path+'"'
                            print('Applying cloud mask.')
                            os.system(gdal_call)


                #remove processed tiles from list

                print('')
                print('Removing tile '+tile_name+' from original list.')
                print('')
                print('')
                tile_index=last_tiles.index(tile_name)
                last_tiles.pop(tile_index)

            print ('Searching tiles: '+str(last_tiles))

    elif date_first=='0' and date_last=='0':
        for folder in imagery_directory_content:
            difference_NBR(date_last=folder, date_first='0')

    else:
        print ('Date folder not found. Date from: '+date_first+'. Date to: '+date_last)




def vector(date_folder):   #Converts differential NBR rasters to discrete values and to vector files
    NBRdiff_threshold=0.6             #higher thresholds means more data being dismissed
    thr_2=str(0.2+NBRdiff_threshold)
    thr_3=str(0.4+NBRdiff_threshold)
    thr_4=str(0.8+NBRdiff_threshold)


    NBR_threshold=-0.15
    NBR_2=-0.2
    NBR_3=-0.25
    NBR_4=-0.3



    imagery_directory_content=os.listdir(imagery_directory)
    if date_folder in imagery_directory_content:    #search the folder for a given date
        date_folder_directory=os.path.join(imagery_directory, str(date_folder))
        date_folder_content=os.listdir(date_folder_directory)
    else:
        print ('Folder '+date_folder+' not found')
        return

    for files in date_folder_content:
        file_name=files.split('.')[0]
        file_format=files.split('.')[-1]
        file_name_parts=file_name.split('_')
        if file_name_parts[-1]=='NBR' and file_name_parts[-2]=='DIF' and file_format=='tif':         #if NBR_diff file is found, process
            print('')
            print('Calculating NBR diff VECTOR.')
            NBR_file_path=os.path.join(date_folder_directory, files)
            output_temp_vector_name=str(file_name)+'_temp.tif'
            output_temp_vector_path=os.path.join(date_folder_directory,output_temp_vector_name)
            output_vector_name=str(file_name)+'.shp'
            output_vector_path=os.path.join(date_folder_directory,output_vector_name)
            if output_vector_name in date_folder_content:
                print('Output file already exists.')
            else:
                print('Creating temporary raster: ', output_temp_vector_path)
                gdal_call='gdal_calc.py -A "'+NBR_file_path+'"  --outfile="'+output_temp_vector_path+'" --calc="1*(('+str(NBRdiff_threshold)+'<=A)*(A<'+thr_2+'))+2*(('+thr_2+'<=A)*(A<'+thr_3+'))+3*(('+thr_3+'<=A)*(A<'+thr_4+'))+4*(A>='+thr_4+')*(A<20)+200*(A==200)" --NoDataValue=0 --type="Byte"'

                os.system(gdal_call)

                print('')
                print('')
                print('Creating vector file: ', output_vector_path)

                date_to_formt=str(date_folder)[:4]+'-'+str(date_folder)[4:6]+'-'+str(date_folder)[6:8]

                date_from_formt=str(file_name_parts[2])[:4]+'-'+str(file_name_parts[2])[4:6]+'-'+str(file_name_parts[2])[6:8]


                polygonize_call='gdal_polygonize.py "'+output_temp_vector_path+'" -f "ESRI Shapefile" "'+output_vector_path+'"'
                os.system(polygonize_call)
                add_column_call=('ogrinfo "'+output_vector_path+'" -sql "ALTER TABLE '+str(file_name)+' ADD COLUMN date_from date "')
                os.system(add_column_call)
                set_default_sql=('ogrinfo "'+output_vector_path+'" -dialect SQLite -sql "UPDATE '+str(file_name)+' SET date_from=\''+date_from_formt+'\' "')
                os.system(set_default_sql)
                add_column2_call=('ogrinfo "'+output_vector_path+'" -sql "ALTER TABLE '+str(file_name)+' ADD COLUMN date_to date "')
                os.system(add_column2_call)
                set_default2_sql=('ogrinfo "'+output_vector_path+'" -dialect SQLite -sql "UPDATE '+str(file_name)+' SET date_to=\''+date_to_formt+'\' "')
                os.system(set_default2_sql)
                print (set_default2_sql)
                os.remove(output_temp_vector_path)


        if file_name_parts[-1]=='NBR' and file_name_parts[-2]!='DIF' and file_format=='tif':         #if NBR_diff file is found, process
            print('')
            print('Calculating NBR VECTOR.')

            NBR_file_path=os.path.join(date_folder_directory, files)
            output_temp_vector_name=str(file_name)+'_temp.tif'
            output_temp_vector_path=os.path.join(date_folder_directory,output_temp_vector_name)
            output_vector_name=str(file_name)+'.shp'
            output_vector_path=os.path.join(date_folder_directory,output_vector_name)
            if output_vector_name in date_folder_content:
                print('Output file already exists.')
            else:
                print('Creating temporary raster: ', output_temp_vector_path)
                gdal_call='gdal_calc.py -A "'+NBR_file_path+'"  --outfile="'+output_temp_vector_path+'" --calc="1*(('+str(NBR_threshold)+'>=A)*(A>'+str(NBR_2)+'))+2*(('+str(NBR_2)+'>=A)*(A>'+str(NBR_3)+'))+3*(('+str(NBR_3)+'>=A)*(A>'+str(NBR_4)+'))+4*(A<='+str(NBR_4)+')*(A>-10)+200*(A==200)" --NoDataValue=0 --type="Byte"'

                os.system(gdal_call)

                print('')
                print('')
                print('Creating vector file: ', output_vector_path)

                date_formt=str(date_folder)[:4]+'-'+str(date_folder)[4:6]+'-'+str(date_folder)[6:8]




                polygonize_call='gdal_polygonize.py "'+output_temp_vector_path+'" -f "ESRI Shapefile" "'+output_vector_path+'"'
                os.system(polygonize_call)
                add_column2_call=('ogrinfo "'+output_vector_path+'" -sql "ALTER TABLE '+str(file_name)+' ADD COLUMN date_ date "')
                os.system(add_column2_call)
                set_default2_sql=('ogrinfo "'+output_vector_path+'" -dialect SQLite -sql "UPDATE '+str(file_name)+' SET date_=\''+date_formt+'\' "')
                os.system(set_default2_sql)
                print (set_default2_sql)
                os.remove(output_temp_vector_path)




def load_postgis(date_folder):


    #Load Pstgis access data from file
    auth_file_path=os.path.join(system_directory,r'postgis_data.txt')

    try:
        auth_file=open(auth_file_path, 'r')
        username=auth_file.readline().strip('\n')
        password=auth_file.readline().strip('\n')
        db_host=auth_file.readline().strip('\n')
        db_name=auth_file.readline().strip('\n')
        auth_file.close()
    except:
        raise CustomError('No postgis_data.txt file in main folder or corrupted file. Format: 1st line username, 2nd line password, 3rd line host')


    print('Loading Vector files to PostGIS.')
    imagery_directory_content=os.listdir(imagery_directory)
    if date_folder in imagery_directory_content:    #search the folder for a given date
        date_folder_directory=os.path.join(imagery_directory, str(date_folder))
        date_folder_content=os.listdir(date_folder_directory)
        uploaded_file_path=os.path.join(date_folder_directory,r'this_folder_was_uploaded.txt')

    else:
        print ('Folder not found')
        return
    for files in date_folder_content:
        if files == 'this_folder_was_uploaded.txt':  #if beacon file found, do not upload
            print('File was already uploaded. If you want to force upload, delete file: '+uploaded_file_path+' and try again')
            return('File was already uploaded. If you want to force upload, delete file: '+uploaded_file_path+' and try again')

    for files in date_folder_content:
        file_name=files.split('.')[0]
        file_format=files.split('.')[-1]
        file_name_parts=file_name.split('_')
        if file_name_parts[-1]=='NBR' and file_name_parts[-2]=='DIF' and file_format=='shp':
            shape_file_path=os.path.join(date_folder_directory, files)
            print('')
            print('Current File: ',shape_file_path)
            ogr_call='ogr2ogr -append -f "PostgreSQL" PG:"host='+db_host+' user='+username+' dbname='+db_name+' password='+password+'" "'+shape_file_path+'" -nln NBR_DIFF'
            os.system(ogr_call)
            f= open(uploaded_file_path,"w+")  #leave beacon file
            f.close()
            print('Database uploaded')
        if file_name_parts[-1]=='NBR' and file_name_parts[-2]!='DIF' and file_format=='shp':
            shape_file_path=os.path.join(date_folder_directory, files)
            print('')
            print('Current File: ',shape_file_path)
            ogr_call='ogr2ogr -append -f "PostgreSQL" PG:"host='+db_host+' user='+username+' dbname='+db_name+' password='+password+'" "'+shape_file_path+'" -nln NBR'
            os.system(ogr_call)
            f= open(uploaded_file_path,"w+")  #leave beacon file
            f.close()
            print('Database uploaded')
    print('Done')
    print('')






